<?php

/*
 * Copyright 2005 - 2025 Centreon (https://www.centreon.com/)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * For more information : contact@centreon.com
 *
 */

declare(strict_types=1);

namespace Tests\App\ActivityLogging\Infrastructure\Double;

use App\ActivityLogging\Domain\Aggregate\ActivityLog;
use App\ActivityLogging\Domain\Aggregate\ActivityLogId;
use App\ActivityLogging\Domain\Repository\ActivityLogRepository;

final class FakeActivityLogRepository implements ActivityLogRepository
{
    /** @var array<int, ActivityLog> */
    public array $activityLogs = [];

    public function add(ActivityLog $activityLog): void
    {
        do {
            $id = mt_rand();
        } while (isset($this->activityLogs[$id]));

        $reflection = new \ReflectionProperty($activityLog, 'id');
        $reflection->setAccessible(true);
        $reflection->setValue($activityLog, new ActivityLogId($id));

        $this->activityLogs[$id] = $activityLog;
    }

    public function count(): int
    {
        return count($this->activityLogs);
    }

    public function find(ActivityLogId $id): ?ActivityLog
    {
        return $this->activityLogs[$id->value] ?? null;
    }
}
