export const searchLabel = 'Search';
export const labelOpen = 'Open';
export const labelClear = 'Clear';

export const labelSelectAll = 'Select all';
export const labelUnSelectAll = 'Unselect all';
export const labelElementsFound = '{{total}} element(s) found';
